@@ -8,20 +8,31 @@
 
 __all__ = ["convolution_stft", "get_same_padding", "conv1d_decoder"]
 
+import math
+
 import torch
 import torch.nn.functional as F
 from torch import Tensor, nn
 
 
+def _fourier_matrix(n_fft: int, device: torch.device) -> torch.Tensor:
+    # https://mathworld.wolfram.com/FourierMatrix.html
+    idx = torch.arange(0, n_fft, device=device, dtype=torch.float).unsqueeze(1)
+    z = idx @ idx.T
+    imaginary = -2 * math.pi * z / n_fft
+    real = torch.zeros_like(imaginary)
+    return torch.exp(torch.complex(real, imaginary))
+
+
 def convolution_stft(
-    input_data,
+    input_data: torch.Tensor,
     n_fft: int = 1024,
     hop_length: int = 512,
     win_length: int = 1024,
     window: torch.Tensor = torch.hann_window(1024, periodic=False),
     center: bool = True,
     return_complex: bool = False,
-):
+) -> torch.Tensor:
     """Implements the stft operation using the convolution method. This is one alternative
     to make possible to export code using this operation to onnx and arm based environments.
     The signature shuld follow the same as torch.stft, making it possible to just swap the two.
@@ -29,8 +40,9 @@ def convolution_stft(
     """
     assert n_fft >= win_length
     pad_amount = int(n_fft / 2)
+    window = window.to(input_data.device)
 
-    fourier_basis = torch.fft.fft(torch.eye(n_fft))
+    fourier_basis = _fourier_matrix(n_fft, device=input_data.device)
 
     cutoff = int((n_fft / 2 + 1))
     fourier_basis = torch.vstack(
