@@ -3,57 +3,95 @@
 
 # Copyright (c) 2021 scart97
 
-__all__ = ["Wav2Vec2Module"]
+__all__ = [
+    "Wav2Vec2Module",
+    "TextTransformConfig",
+    "ModelConfig",
+    "OptimizerConfig",
+]
 
+from copy import copy
+from dataclasses import dataclass, field
 from typing import Any, Dict, List, Tuple
 
 import pytorch_lightning as pl
 import torch
 from torch import Tensor, nn
-from torch.nn.functional import ctc_loss, log_softmax
-from transformers import Wav2Vec2Model
 
+try:
+    from transformers import Wav2Vec2Model
+except ModuleNotFoundError as transformers_not_installed:
+    raise ImportError(
+        "To use wav2vec please install the transformers extension, by calling `pip install thunder-speech[transformers]`"
+    ) from transformers_not_installed
+
+from torchaudio.models.wav2vec2.utils.import_huggingface import _get_config, _get_model
+
+from thunder.blocks import linear_decoder
+from thunder.ctc_loss import calculate_ctc
 from thunder.metrics import CER, WER
-from thunder.text_processing.transform import BatchTextTransformer, Vocab
+from thunder.text_processing.transform import BatchTextTransformer, TextTransformConfig
 from thunder.wav2vec.transform import Wav2Vec2Preprocess
 
 
+@dataclass
+class ModelConfig:
+    """Configuration to create the wav2vec 2.0 encoder.
+
+    Attributes:
+        model_name: Name of the original huggingface checkpoint to load from. defaults to 'facebook/wav2vec2-base'
+        gradient_checkpointing: Use gradient checkpointing to save memory at the expense of slower backward pass. defaults to False.
+        additional_kwargs: Any other option that can be passed to the original Wav2Vec2Model.from_pretrained. defaults to {}.
+        decoder_dropout: Dropout before the final decoding layer. defaults to 0.1.
+    """
+
+    model_name: str = "facebook/wav2vec2-base"
+    gradient_checkpointing: bool = False
+    additional_kwargs: Dict[str, Any] = field(default_factory=lambda: copy({}))
+    decoder_dropout: float = 0.1
+
+
+@dataclass
+class OptimizerConfig:
+    """Configuration used by the optimizer
+
+    Attributes:
+        learning_rate: learning rate. defaults to 3e-4.
+    """
+
+    learning_rate: float = 3e-4
+
+
 class Wav2Vec2Module(pl.LightningModule):
     def __init__(
         self,
-        initial_vocab_tokens: List[str],
-        model_name: str = "facebook/wav2vec2-base",
-        gradient_checkpointing: bool = False,
-        decoder_dropout: float = 0.1,
-        learning_rate: float = 3e-4,
-        **kwargs: Dict[str, Any],
+        text_cfg: TextTransformConfig,
+        encoder_cfg: ModelConfig = ModelConfig(),
+        optim_cfg: OptimizerConfig = OptimizerConfig(),
     ):
         """Wav2Vec model for fine-tuning.
 
         Args:
-            initial_vocab_tokens : List of tokens to be used in the vocab, special tokens should not be included here. Check [`docs`](https://scart97.github.io/thunder-speech/quick%20reference%20guide/#how-to-get-the-initial_vocab_tokens-from-my-dataset)
-            model_name : Name of the original huggingface checkpoint to load from.
-            gradient_checkpointing : Use gradient checkpointing to save memory at the expense of slower backward pass.
-            decoder_dropout : Dropout before the final decoding layer
-            learning_rate : Learning rate used on the optimizer.
-            kwargs: Any other option that can be passed to the original Wav2Vec2Model.from_pretrained
+            text_cfg: Configuration for the text processing pipeline
+            encoder_cfg: Configuration for the wav2vec encoder
+            optim_cfg: Configuration for the optimizer used during training
         """
         super().__init__()
         self.save_hyperparameters()
         self.audio_transform = Wav2Vec2Preprocess()
 
         self.encoder = Wav2Vec2Model.from_pretrained(
-            model_name,
-            gradient_checkpointing=gradient_checkpointing,
-            **kwargs,
+            encoder_cfg.model_name,
+            gradient_checkpointing=encoder_cfg.gradient_checkpointing,
+            **encoder_cfg.additional_kwargs,
         )
         self.encoder.feature_extractor._freeze_parameters()
 
-        self.text_transform = self.build_text_transform(initial_vocab_tokens)
-        self.decoder = self.build_decoder(
-            decoder_dropout,
+        self.text_transform = BatchTextTransformer(text_cfg)
+        self.decoder = linear_decoder(
             self.encoder.config.hidden_size,
             len(self.text_transform.vocab),
+            encoder_cfg.decoder_dropout,
         )
 
         # Metrics
@@ -62,38 +100,6 @@ class Wav2Vec2Module(pl.LightningModule):
         # Example input is one second of fake audio
         self.example_input_array = torch.randn((10, 16000))
 
-    def build_text_transform(
-        self, initial_vocab_tokens: List[str]
-    ) -> BatchTextTransformer:
-        """Overwrite this function if you want to change how the text processing happens inside the model.
-
-        Args:
-            initial_vocab_tokens : List of tokens to create the vocabulary, special tokens should not be included here.
-
-        Returns:
-            The transform that will both `encode` the text and `decode_prediction`.
-        """
-        vocab = Vocab(initial_vocab_tokens)
-        return BatchTextTransformer(vocab=vocab)
-
-    def build_decoder(
-        self, decoder_dropout: float, decoder_input_channels: int, vocab_size: int
-    ) -> nn.Module:
-        """Overwrite this function if you want to change the model decoder.
-
-        Args:
-            decoder_dropout: Amount of dropout to be used in the decoder
-            decoder_input_channels : Number of input channels of the decoder. That is the number of channels of the features created by the encoder.
-            vocab_size : Number of output classes
-
-        Returns:
-            Module that represents the decoder.
-        """
-        return nn.Sequential(
-            nn.Dropout(decoder_dropout),
-            nn.Linear(decoder_input_channels, vocab_size),
-        )
-
     def forward(self, audio: Tensor) -> Tensor:
         """Process the audio tensor to create the probabilities.
 
@@ -105,10 +111,7 @@ class Wav2Vec2Module(pl.LightningModule):
         """
         features = self.audio_transform(audio)
         encoded_dict = self.encoder(features)
-        probs = self.decoder(encoded_dict.last_hidden_state)
-        # Change from (batch, time, #vocab) to (batch, #vocab, time)
-        # that is expected by the rest of the library
-        return probs.permute(0, 2, 1)
+        return self.decoder(encoded_dict.last_hidden_state)
 
     @torch.jit.export
     def predict(self, x: Tensor) -> List[str]:
@@ -123,25 +126,6 @@ class Wav2Vec2Module(pl.LightningModule):
         pred = self(x)
         return self.text_transform.decode_prediction(pred.argmax(1))
 
-    def calculate_loss(self, probabilities, y, prob_lens, y_lens):
-        # Change from (batch, #vocab, time) to (time, batch, #vocab)
-        probabilities = probabilities.permute(2, 0, 1)
-        logprobs = log_softmax(probabilities, dim=2)
-        # Calculate the logprobs correct length based on the
-        # normalized original lengths
-        prob_lens = (prob_lens * logprobs.shape[0]).long()
-        blank = self.text_transform.vocab.blank_idx
-
-        return ctc_loss(
-            logprobs,
-            y,
-            prob_lens,
-            y_lens,
-            blank=blank,
-            reduction="mean",
-            zero_infinity=True,
-        )
-
     def training_step(
         self, batch: Tuple[torch.Tensor, torch.Tensor, List[str]], batch_idx: int
     ) -> torch.Tensor:
@@ -158,7 +142,9 @@ class Wav2Vec2Module(pl.LightningModule):
         y, y_lens = self.text_transform.encode(texts, device=self.device)
 
         probabilities = self(audio)
-        loss = self.calculate_loss(probabilities, y, audio_lens, y_lens)
+        loss = calculate_ctc(
+            probabilities, y, audio_lens, y_lens, self.text_transform.vocab.blank_idx
+        )
 
         self.log("loss/train_loss", loss)
         return loss
@@ -179,7 +165,9 @@ class Wav2Vec2Module(pl.LightningModule):
         y, y_lens = self.text_transform.encode(texts, device=self.device)
 
         probabilities = self(audio)
-        loss = self.calculate_loss(probabilities, y, audio_lens, y_lens)
+        loss = calculate_ctc(
+            probabilities, y, audio_lens, y_lens, self.text_transform.vocab.blank_idx
+        )
 
         decoded_preds = self.text_transform.decode_prediction(probabilities.argmax(1))
         decoded_targets = self.text_transform.decode_prediction(y)
@@ -199,5 +187,73 @@ class Wav2Vec2Module(pl.LightningModule):
         """
         return torch.optim.Adam(
             filter(lambda p: p.requires_grad, self.parameters()),
-            lr=self.hparams.learning_rate,
+            lr=self.hparams.optim_cfg.learning_rate,
         )
+
+
+class Wav2Vec2Scriptable(nn.Module):
+    def __init__(self, module: Wav2Vec2Module, quantized: bool = False):
+        """Wav2vec model ready to be jit scripted and used in inference.
+        This class is necessary because the torchaudio imported model is not
+        a drop in replacement of the transformers implementation, causing some
+        trouble with the jit.
+
+        Args:
+            module : The trained Wav2Vec2 module that you want to export
+            quantized : Controls if quantization will be applied to the model.
+        """
+        super().__init__()
+        # Transforming model to torchaudio one
+        encoder_config = _get_config(module.encoder.config)
+        encoder_config["encoder_num_out"] = len(module.text_transform.vocab)
+        imported = _get_model(**encoder_config)
+        imported.feature_extractor.load_state_dict(
+            module.encoder.feature_extractor.state_dict()
+        )
+        imported.encoder.feature_projection.load_state_dict(
+            module.encoder.feature_projection.state_dict()
+        )
+        imported.encoder.transformer.load_state_dict(
+            module.encoder.encoder.state_dict()
+        )
+        imported.encoder.readout.load_state_dict(module.decoder[1].state_dict())
+
+        if quantized:
+            imported.encoder.transformer.pos_conv_embed.__prepare_scriptable__()
+            imported = torch.quantization.quantize_dynamic(
+                imported, qconfig_spec={torch.nn.Linear}, dtype=torch.qint8
+            )
+
+        self.model = imported
+
+        self.audio_transform = module.audio_transform
+        self.text_transform = module.text_transform
+
+    def forward(self, audio: Tensor) -> Tensor:
+        """Process the audio tensor to create the probabilities.
+
+        Args:
+            audio : Audio tensor of shape [batch_size, time]
+
+        Returns:
+            Tensor with the prediction probabilities.
+        """
+        features = self.audio_transform(audio)
+        outputs = self.model(features)
+        probs = outputs[0]
+        # Change from (batch, time, #vocab) to (batch, #vocab, time)
+        # that is expected by the rest of the library
+        return probs.permute(0, 2, 1)
+
+    @torch.jit.export
+    def predict(self, x: Tensor) -> List[str]:
+        """Use this function during inference to predict.
+
+        Args:
+            x : Audio tensor of shape [batch_size, time]
+
+        Returns:
+            A list of strings, each one contains the corresponding transcription to the original batch element.
+        """
+        pred = self(x)
+        return self.text_transform.decode_prediction(pred.argmax(1))
